/*

    	Find the "luminosity centre" of an image, examining
	only pixels brighter than half the maximum luminosity
	in the image.  The centre determined in this manner is
	written to standard output as two numbers on a single
	line.
	
	Input is a raw PPM file which is expected to be in
	the precise format written by the NETPBM utilities.
	
	What's this good for?  One thing: automating the
	cropping of images of celestial bodies from
	image frames.  But if you have more than 1200 such
	images to crop (as in the Transit of Venus movie which
	motivated the creation of this little tool), it's a
	lot easier than doing it by hand.
	
	Perpetrated on the Ides of June 2004 by John Walker,
	http://www.fourmilab.ch/
	
*/


#define VERBOSE

#include <stdio.h>
#include <stdlib.h>

int main()
{
    char s[132];
    int width, height, maxval;
    unsigned char *pixels;
    int x, y, vcent = -1, hcent = -1;
    int totalum = 0, ilum, lthresh;

#define R   0
#define G   1
#define B   2

#define Px(x, y, c)    (((((y) * width) + (x)) * 3) + c)
#define P(x, y, c)     pixels[Px(x, y, c)]

#define Luma(x, y)	(((P(x, y, R) * 299) + (P(x, y, G) * 587) + (P(x, y, B) * 114)) / 1000)
    
    fgets(s, sizeof s, stdin);	    /* Throw away "magic" file type line */
    fgets(s, sizeof s, stdin);	    /* Read image width and height */
    sscanf(s, "%d %d", &width, &height);
    fgets(s, sizeof s, stdin);	    /* Read pixel maximum value. */
    sscanf(s, "%d", &maxval);
    
#ifdef VERBOSE
    fprintf(stderr, "%d by %d, maxval %d\n", width, height, maxval);
#endif

    pixels = malloc(Px(0, height, 0));
    
    fread(pixels, Px(0, height, 0), 1, stdin);
    
    lthresh = maxval / 2;
    
    for (x = 0; x < width; x++) {
    	for (y = 0; y < height; y++) {
	    if (Luma(x, y) > lthresh) {
	    	totalum += Luma(x, y);
	    }
	}
    }
    
#ifdef VERBOSE
    fprintf(stderr, "Total luminosity: %d\n", totalum);
#endif

    totalum /= 2;
   
    ilum = 0;
    for (x = 0; x < width; x++) {
    	for (y = 0; y < height; y++) {
	    if (Luma(x, y) > lthresh) {
	    	ilum += Luma(x, y);
	    }
	}
	if (ilum >= totalum) {
	    hcent = x;
	    break;
	}
    }
   
    ilum = 0;
    for (y = 0; y < height; y++) {
    	for (x = 0; x < width; x++) {
	    if (Luma(x, y) > lthresh) {
	    	ilum += Luma(x, y);
	    }
	}
	if (ilum >= totalum) {
	    vcent = y;
	    break;
	}
    }

#ifdef VERBOSE
    fprintf(stderr, "Luminosity centre: %d, %d\n", hcent, vcent);
#endif
    
    /*	Now print the determined centre point.  That's it.  */
    
    printf("%d %d\n", hcent, vcent);

    return 0;
}
